--
-- GuildArtisanWindow
--
-- Copyright (c) 2021 Dana Basken
--

local AceEvent = LibStub("AceEvent-3.0")
local AceTimer = LibStub("AceTimer-3.0")

GuildArtisanWindow = {}
GuildArtisanWindow.__index = GuildArtisanWindow

function GuildArtisanWindow:Create(db, cache)
  local instance = {}
  setmetatable(instance, GuildArtisanWindow)
  instance:Construct(db, cache)
  return instance
end

function GuildArtisanWindow:Construct(db, cache)
  self.db = db
  self.cache = cache
  self.needsUpdate = false
  self.guildInfo = {}

  self:GuildRefresh()

  if not self.db.char.windowPosition then self.db.char.windowPosition = {relative = "CENTER", x = 0, y = 0} end

  self:CreateUI()

  AceEvent.RegisterMessage(self, "GUILD_ARTISAN_CACHE_UPDATE")

  AceEvent.RegisterEvent(self, "GUILD_ROSTER_UPDATE")

  AceTimer.ScheduleRepeatingTimer(self, "PeriodicTasks", 1)
  AceTimer.ScheduleRepeatingTimer(self, "GuildRefreshTask", 120)
end

function GuildArtisanWindow:CreateUI()
  self:CreateWindow()
  self:CreateTitleBar()
  if self.db.char.windowVisible then self:Show() else self:Hide() end
  self.rootRow = GuildArtisanRow:Create(nil, nil, self.window.content)
end

function GuildArtisanWindow:CreateWindow()
  self.window = CreateFrame("Frame", "GuildArtisanWindow", UIParent, BackdropTemplateMixin and "BackdropTemplate")
  self.window:SetFrameStrata("DIALOG")
  self.window:SetWidth(400)
  self.window:SetHeight(400)
  self.window:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 32, insets = {left = 10, right = 10, top = 10, bottom = 10}})
  self.window:SetPoint(self.db.char.windowPosition.relative, self.db.char.windowPosition.x, self.db.char.windowPosition.y)
  self.window:SetBackdropColor(0.2, 0, 0.2, 1)

  self.window:RegisterForDrag("LeftButton")
  self.window:SetScript("OnDragStart", self.window.StartMoving)
  self.window:SetScript("OnDragStop", function()
    self.window:StopMovingOrSizing()
    local point, relativeTo, relativePoint, xOfs, yOfs = self.window:GetPoint()
    self.db.char.windowPosition = {relative = relativePoint, x = xOfs, y = yOfs}
  end)

  self.window.scroll = CreateFrame("ScrollFrame", nil, self.window, "UIPanelScrollFrameTemplate")
  self.window.content = CreateFrame("Frame", "$parentContent", nil, BackdropTemplateMixin and "BackdropTemplate")
  self.window.scroll:SetScrollChild(self.window.content)
  self.window.scroll:SetPoint("TOPLEFT", self.window ,"TOPLEFT", 10, -32)
  self.window.scroll:SetPoint("BOTTOMRIGHT", self.window ,"BOTTOMRIGHT", -32, 10)
  self.window.content:SetSize(self.window.scroll:GetWidth(), 0)
end

function GuildArtisanWindow:CreateTitleBar()
  self.titleBar = CreateFrame("Frame", "$parentTitleBar", self.window, BackdropTemplateMixin and "BackdropTemplate")
  self.titleBar:SetWidth(400)
  self.titleBar:SetHeight(42)
  self.titleBar:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 32, insets = {left = 10, right = 10, top = 10, bottom = 10}})
  self.titleBar:SetPoint("TOPLEFT", 0, 0)
  self.titleBar:SetBackdropColor(0.1, 0, 0.1, 1)

  local close = CreateFrame("Button","$parentClose", self.titleBar)
  close:SetHeight(24)
  close:SetWidth(24)
  close:SetPoint("TOPRIGHT", -10, -10)
  close:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
  close:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
  close:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight", "ADD")
  close:SetScript("OnClick", function() self:Hide() end)

  self.lock = CreateFrame("Button","$parentLock", self.titleBar)
  self.lock:SetHeight(24)
  self.lock:SetWidth(24)
  self.lock:SetPoint("TOPRIGHT", -30, -10)
  self.lock:SetPushedTexture("Interface\\Buttons\\LockButton-Unlocked-Down")
  self.lock:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight", "ADD")
  self.lock:SetScript("OnClick", function() self:ToggleLock() end)
  self:UpdateLockButton()

  self.window.nameFrame = self.titleBar:CreateFontString("FontString", "OVERLAY")
  self.window.nameFrame:SetFont("Fonts\\FRIZQT__.TTF", 10, "MONOCHROME")
  self.window.nameFrame:SetPoint("CENTER", self.titleBar, 0, 0)
  self:UpdateTitle()
end

function GuildArtisanWindow:PeriodicTasks()
  if self.needsUpdate then
    self.needsUpdate = false
    if self.db.char.windowVisible then
      self:UpdateRecipes()
      self:UpdateUI()
    end
  end
end

function GuildArtisanWindow:GuildRefreshTask()
  if self.db.char.windowVisible then
    self:GuildRefresh()
  end
end

function GuildArtisanWindow:GuildRefresh()
  C_GuildInfo.GuildRoster()
  local count, _, _ = GetNumGuildMembers();
  for index = 1, count do
    name, _, _, _, classDisplay, _, _, _, online, _, class, _, _, mobile, _, _, GUID = GetGuildRosterInfo(index)
    if name then
      self.guildInfo[name] = {online = online, classDisplay = classDisplay, class = class, mobile = mobile, GUID = GUID, index = index}
    end
  end
end

function GuildArtisanWindow:GetOrCreateRow(name, icon, parent, frame, data, renderer, updater)
  local child = parent:GetChildNamed(name)
  if child then return child end
  child = GuildArtisanRow:Create(name, icon, frame, data, renderer, updater)
  parent:AddChild(child)
  return child
end

function GuildArtisanWindow:UpdateRecipes()
  if self.cache:IsReady() then

    local remainingProfessions = self.rootRow:GetChildNames()

    for _, profession in pairs(self.cache:GetProfessions()) do

      local recipes = self.cache:GetRecipesForProfession(profession)
      local icon = GuildArtisan_Config.EXPANSIONS[self.db.char.expansion].icons[profession]
      local data = {count = #recipes}
      local professionRow = self:GetOrCreateRow(profession, icon, self.rootRow, self.rootRow.frame, data, self.ProfressionRowSetup, self.ProfressionRowUpdate)
      professionRow:UpdateData(data)
      remainingProfessions[profession] = false

      for _, recipe in pairs(recipes) do

        local maxPlayer = nil
        local recipeRow = self:GetOrCreateRow(recipe.name, recipe.recipeId, professionRow, professionRow.frame, nil, self.RecipeRowSetup, self.RecipeRowUpdate)
        local players = self.cache:GetPlayersForRecipe(recipe)

        for _, player in pairs(players) do

          if not maxPlayer or player.recipe.level > maxPlayer.recipe.level then maxPlayer = player end

          local name, realm = strsplit("-", player.name)
          if not realm then player.name = ("%s-%s"):format(name, GetRealmName()) end

          local guildPlayer = self.guildInfo[player.name]
          local icon = ("Interface/Icons/ClassIcon_%s"):format(guildPlayer.class)
          local playerRow = self:GetOrCreateRow(player.name, icon, recipeRow, recipeRow.frame, recipe, self.PlayerRowSetup, self.PlayerRowUpdate)
          playerRow:UpdateData(recipe)

        end

        if maxPlayer then
          recipeRow:UpdateData({level = maxPlayer.recipe.level, name = maxPlayer.name})
        end

      end

    end

    for profession, dangling in pairs(remainingProfessions) do
      if dangling then
        local row = self.rootRow:GetChildNamed(profession)
        self.rootRow:RemoveChild(row)
      end
    end

    self.rootRow:UpdateUI()
  else
    self:RefreshUI()
  end
end

function GuildArtisanWindow:ProfressionRowSetup(frame, data)
  frame.profession = frame:CreateFontString("FontString", "OVERLAY")
  frame.profession:SetFont("Fonts\\FRIZQT__.TTF", 10, "MONOCHROME")
  frame.profession:SetTextColor(1.0, 1.0, 1.0, 0.5)
  frame.profession:SetPoint("LEFT", frame, 0, 0)
  frame:SetWidth(100)
end

function GuildArtisanWindow:ProfressionRowUpdate(frame, data)
  local recipes = ""
  if data.count == 1 then recipes = "recipe" else recipes = "recipes" end
  frame.profession:SetText(("(%d %s)"):format(data.count, recipes))
end

function GuildArtisanWindow:RecipeRowSetup(frame, data)
  frame.recipe = frame:CreateFontString("FontString", "OVERLAY")
  frame.recipe:SetFont("Fonts\\FRIZQT__.TTF", 10, "MONOCHROME")
  frame.recipe:SetTextColor(1.0, 1.0, 1.0, 0.5)
  frame.recipe:SetPoint("LEFT", frame, 0, 0)
  frame:SetWidth(100)
end

function GuildArtisanWindow:RecipeRowUpdate(frame, data)
  local text = ""
  local name, realm = strsplit("-", data.name)
  if data.level > 0 then text = ("(rank %d: %s)"):format(data.level, name) end
  frame.recipe:SetText(text)
end

function GuildArtisanWindow:PlayerRowSetup(frame, data)
  frame.rank = CreateFrame("Frame", nil, frame)
  frame.rank:SetWidth(100)
  frame.rank:SetHeight(frame:GetHeight())
  frame.rank:SetPoint("LEFT", 0, 0)
  frame.rank.bar = ProgressBar:Create(frame.rank, data.currentXp, data.requiredXp)
  frame:SetWidth(100)
end

function GuildArtisanWindow:PlayerRowUpdate(frame, data)
  if data.requiredXp == 0 then
    frame.rank.bar:Set(("rank %d (max)"):format(data.level), 100, 100)
  else
    local percentage = math.floor((data.currentXp / data.requiredXp) * 100)
    frame.rank.bar:Set(("rank %s (%d%%)"):format(data.level, percentage), data.currentXp, data.requiredXp)
  end
end

function GuildArtisanWindow:UpdateTitle()
  local title = "Guild Artisan"
  if self.guildName then
    title = ("%s - %s"):format(title, self.guildName)
  end
  self.window.nameFrame:SetText(title)
end

function GuildArtisanWindow:UpdateUI()
  if self.db.char.windowVisible then
    self.window.content:SetSize(self.window.scroll:GetWidth(), self.rootRow:GetHeight())
    self:UpdateTitle()
  end
end

function GuildArtisanWindow:RefreshUI()
  self.needsUpdate = true
end

function GuildArtisanWindow:GUILD_ARTISAN_CACHE_UPDATE(...)
  self:RefreshUI()
end

function GuildArtisanWindow:GUILD_ROSTER_UPDATE(...)
  self:GuildRefresh()
  self:RefreshUI()
end

function GuildArtisanWindow:Show()
  self.window:Show()
  self.db.char.windowVisible = true
  self:RefreshUI()
end

function GuildArtisanWindow:Hide()
  self.db.char.windowVisible = false
  self.window:Hide()
end

function GuildArtisanWindow:Toggle()
  if self.db.char.windowVisible then
    self:Hide()
  else
    self:Show()
  end
end

function GuildArtisanWindow:Lock()
  self.db.char.windowLocked = true
  self:UpdateLockButton()
end

function GuildArtisanWindow:Unlock()
  self.db.char.windowLocked = false
  self:UpdateLockButton()
end

function GuildArtisanWindow:UpdateLockButton()
  if self.db.char.windowLocked then
    self.window:SetMovable(false)
    self.window:EnableMouse(false)
    self.lock:SetNormalTexture("Interface\\Buttons\\LockButton-Locked-Up")
    self.window:SetBackdropColor(0.2, 0, 0.2, 0.5)
    self.titleBar:SetBackdropColor(0.1, 0, 0.1, 0.5)
  else
    self.window:SetMovable(true)
    self.window:EnableMouse(true)
    self.lock:SetNormalTexture("Interface\\Buttons\\LockButton-Unlocked-Up")
    self.window:SetBackdropColor(0.2, 0, 0.2, 1)
    self.titleBar:SetBackdropColor(0.1, 0, 0.1, 1)
  end
end

function GuildArtisanWindow:ToggleLock()
  if self.db.char.windowLocked then
    self:Unlock()
  else
    self:Lock()
  end
end

function GuildArtisanWindow:SetGuildName(name)
  self.guildName = name
  self:RefreshUI()
end
